@@ -14,10 +14,14 @@ XDOT DRAWING FUNCTIONS, maybe need to move them somewhere else
 		for now keep them at the bottom
 */
 #include "draw.h"
+#include <cgraph/unreachable.h>
 #include <common/colorprocs.h>
+#include <common/types.h>
+#include <common/utils.h>
 #include "smyrna_utils.h"
 #include <glcomp/glutils.h>
 #include <math.h>
+#include <stdlib.h>
 
 #include <xdot/xdot.h>
 #include "viewport.h"
@@ -118,17 +122,16 @@ static void set_options(int param)
 static void DrawBeziers(sdot_op* o, int param)
 {
     int filled;
-    int i = 0;
     xdot_op *  op=&o->op;
     xdot_point* ps = op->u.bezier.pts;
-    view->Topview->global_z = view->Topview->global_z + o->layer*LAYER_DIFF;
+    view->Topview->global_z += o->layer * LAYER_DIFF;
 
     if (op->kind == xd_filled_bezier)
 	filled = 1;
     else
 	filled = 0;
 
-    for (i = 1; i < op->u.bezier.cnt; i += 3) {
+    for (size_t i = 1; i < op->u.bezier.cnt; i += 3) {
 	DrawBezier(ps, filled, param);
 	ps += 3;
     }
@@ -138,17 +141,15 @@ static void DrawBeziers(sdot_op* o, int param)
 //void DrawEllipse(xdot_point* xpoint,GLfloat xradius, GLfloat yradius,int filled)
 static void DrawEllipse(sdot_op*  o, int param)
 {
-    //to draw a circle set xradius and yradius same values
-    GLfloat x, y, xradius, yradius;
     int i = 0;
     int filled;
     xdot_op * op=&o->op;
-    view->Topview->global_z=view->Topview->global_z+o->layer*LAYER_DIFF;
+    view->Topview->global_z += o->layer * LAYER_DIFF;
     set_options(param);
-    x = op->u.ellipse.x - dx;
-    y = op->u.ellipse.y - dy;
-    xradius = (GLfloat) op->u.ellipse.w;
-    yradius = (GLfloat) op->u.ellipse.h;
+    double x = op->u.ellipse.x - dx;
+    double y = op->u.ellipse.y - dy;
+    double xradius = op->u.ellipse.w;
+    double yradius = op->u.ellipse.h;
     if (op->kind == xd_filled_ellipse) {
 	if (param == 0)
 	    glColor4f(view->fillColor.R, view->fillColor.G,
@@ -179,7 +180,7 @@ static void DrawEllipse(sdot_op*  o, int param)
 	//convert degrees into radians
 	float degInRad = (float) (i * DEG2RAD);
 	glVertex3f((GLfloat) (x + cos(degInRad) * xradius),
-		   (GLfloat) (y + sin(degInRad) * yradius), view->Topview->global_z);
+		   (GLfloat) (y + sin(degInRad) * yradius), (GLfloat)view->Topview->global_z);
     }
     glEnd();
 }
@@ -187,7 +188,7 @@ static void DrawEllipse(sdot_op*  o, int param)
 static void DrawPolygon(sdot_op * o, int param)
 {
     xdot_op *  op=&o->op;
-    view->Topview->global_z=view->Topview->global_z+o->layer*LAYER_DIFF;
+    view->Topview->global_z += o->layer * LAYER_DIFF;
 
     set_options(param);
 
@@ -216,9 +217,8 @@ static void DrawPolygon(sdot_op * o, int param)
 
 static void DrawPolyline(sdot_op* o, int param)
 {
-    int i = 0;
     xdot_op * op=&o->op;
-    view->Topview->global_z=view->Topview->global_z+o->layer*LAYER_DIFF;
+    view->Topview->global_z += o->layer * LAYER_DIFF;
 
     if (param == 0)
 	glColor4f(view->penColor.R, view->penColor.G, view->penColor.B,
@@ -229,10 +229,10 @@ static void DrawPolyline(sdot_op* o, int param)
     set_options(param);
     glLineWidth(view->LineWidth);
     glBegin(GL_LINE_STRIP);
-    for (i = 0; i < op->u.polyline.cnt; i = i + 1) {
+    for (size_t i = 0; i < op->u.polyline.cnt; ++i) {
 	glVertex3f((GLfloat) op->u.polyline.pts[i].x - dx,
 		   (GLfloat) op->u.polyline.pts[i].y - dy,
-		   (GLfloat) op->u.polyline.pts[i].z + view->Topview->global_z);
+		   (GLfloat)(op->u.polyline.pts[i].z + view->Topview->global_z));
     }
     glEnd();
 }
@@ -309,7 +309,7 @@ static void InsertImage(sdot_op * o, int param)
     if(!o->img) {
 	x = o->op.u.image.pos.x;
 	y = o->op.u.image.pos.y;
-	i = o->img = glCompImageNewFile (NULL, x, y, o->op.u.image.name, 0);
+	i = o->img = glCompImageNewFile(x, y, o->op.u.image.name);
 	if (!o->img) {
 	    fprintf (stderr, "Could not open file \"%s\" to read image.\n", o->op.u.image.name);
 	    return;
@@ -320,13 +320,25 @@ static void InsertImage(sdot_op * o, int param)
     }
 }
 
+// see usage in EmbedText
+static int put(void *buffer, const char *s) {
+  char **b = buffer;
+
+  for (; *s != '\0'; ++s) {
+    **b = *s;
+    ++(*b);
+  }
+
+  return 0;
+}
+
 static void EmbedText(sdot_op* o, int param)
 {
 	(void)param;
 
 	GLfloat x,y;
 	glColor4f(view->penColor.R,view->penColor.G,view->penColor.B,view->penColor.A);
-	view->Topview->global_z=view->Topview->global_z+o->layer*LAYER_DIFF+0.05;
+	view->Topview->global_z += o->layer * LAYER_DIFF + 0.05;
 	switch (o->op.u.text.align)
 	{
 		case xd_left:
@@ -338,17 +350,28 @@ static void EmbedText(sdot_op* o, int param)
 		case xd_right:
 			x=o->op.u.text.x - o->op.u.text.width;
 			break;
-
+		default:
+			UNREACHABLE();
 	}
 	y=o->op.u.text.y;
 	if (!o->font)
 	{
-		o->font=glNewFont(
-		view->widgets,
-		xml_string (o->op.u.text.text),
-		&view->penColor,
-		pangotext,
-		font_op->op.u.font.name,font_op->op.u.font.size,0);
+		// allocate a buffer large enough to hold the maximum escaped version of the
+		// text
+		char *escaped = calloc(sizeof(char), strlen(o->op.u.text.text) *
+		                                     sizeof("&#xFFFFFFFF;") + 1);
+		if (escaped == NULL)
+		  return;
+
+		// XML-escape the text
+		const xml_flags_t flags = {.dash = 1, .nbsp = 1};
+		char **ptr = &escaped;
+		(void)xml_escape(o->op.u.text.text, flags, put, ptr);
+
+		o->font = glNewFont(view->widgets, escaped, &view->penColor,
+		                    font_op->op.u.font.name, font_op->op.u.font.size, 0);
+
+		free(escaped);
 	}
 	glCompDrawText3D(o->font,x,y,view->Topview->global_z,o->op.u.text.width,font_op->op.u.font.size);
 
@@ -375,13 +398,13 @@ void drawCircle(float x, float y, float radius, float zdepth)
 {
     int i;
     if (radius < 0.3)
-	radius = (float) 0.4;
+	radius = 0.4f;
     glBegin(GL_POLYGON);
     for (i = 0; i < 360; i = i + 36) {
 	float degInRad = (float) (i * DEG2RAD);
 	glVertex3f((GLfloat) (x + cos(degInRad) * radius),
 		   (GLfloat) (y + sin(degInRad) * radius),
-		   (GLfloat) zdepth + view->Topview->global_z);
+		   (GLfloat)(zdepth + view->Topview->global_z));
     }
 
     glEnd();
